import geopandas as gpd
import pandas as pd
import numpy as np
import cluster_algorithms as calgos
from shapely.geometry import Point, box

# load in the airspace
edges = gpd.read_file('original_graph_data/updated.gpkg', layer='edges')
edges = edges.to_crs('EPSG:28992')
bounds = box(*edges.geometry.total_bounds)

cluster_distances = [1000, 2000, 4000, 8000, 16000]
folder = 'voronoi_graphs'

# load a baseline geopackage with the demand distribution at different random seed
conflog = gpd.read_file('log_gpkgs/conflog_baseline_300_4000_1205330928_ON.gpkg')

# remove everything before 10 mins
mask = conflog['time'] < pd.to_datetime('2024-01-01 00:10:00')
conflog = conflog.loc[~mask].copy() 

# get data of points
x_y_points = conflog.geometry.get_coordinates()
features = np.column_stack((x_y_points['x'], x_y_points['y']))
features = features[np.argsort(features[:, 0])]

for cluster_distance in cluster_distances:

    # do clustering
    optimal_labels = calgos.ward(features, cluster_distance)

    # get number of clusters
    n_clusters = np.max(optimal_labels)+1

    # loop through all of the clusters and create a polygon
    polygon_data = {'flow_group': [], 'geometry': []}
        
    for optimal_label in range(n_clusters):
        label_mask = optimal_labels == optimal_label
        cluster_points = features[label_mask,:]

        centroid_point = Point(np.mean(cluster_points, axis=0))


        polygon_data['flow_group'].append(optimal_label)
        polygon_data['geometry'].append(centroid_point)

    # create geodataframe of points
    point_gs = gpd.GeoSeries(polygon_data['geometry'], index=polygon_data['flow_group'], crs='EPSG:28992')

    # get voronoi
    voronoi_gdf = point_gs.voronoi_polygons(extend_to=bounds)

    # save the gdfs
    output_file =f'{folder}/voronoi{cluster_distance}.gpkg'
    voronoi_gdf.to_file(output_file, layer='voronoi')
    point_gs.to_file(output_file, layer='centroids')

